c
c_________________________________________________________________________
c
c*****currnt
c
c     This subroutine returns the matrix elements of the relativistic single
c     nucleon current operator between positive and negative energy Dirac 
c     spinors.
c
c     11-2-94  modified routine to allow for offshell vertex. This involves
c              splitting the output into pieces proportional to f1 and f2 and
c              leaving the calculation of the form factors to the routine
c              calculating the transition amplitudes. This enlarges the size
c              of the output array.
c
c     INPUT
c           qc...........momentum transfer in frame of calculation
c           omegac.......energy transfer in frame of calculation
c           pp...........momentum of final nucleon
c           thetap.......angle of final nucleon
c           p............momentum of initial nucleon
c           theta........angle of initial nucleon
c           itype=0......photon attaches to particle 1
c                 1......photon attaches to particle 2
c           irel=1.......relativistic current operator
c               =0.......nonrelativistic current operator
c
c     OUTPUT
c           sj(lnucp,lnuc,inrgyp,inrgy,lgam,icur)
c
c               lnucp=0..............positive nucleon helicity
c                    =1..............negative nucleon helicity
c               lnuc=0...............positive nucleon helicity
c                   =1...............negative nucleon helicity
c               inrgyp=0.............positive energy projection
c                     =1.............negative energy projection
c               inrgy=0..............positive energy projection
c                    =1..............negative energy projection
c               lgam=0,1.............photon helicity
c               icur=0...............terms proportional to F_1 or F_3
c                   =1...............terms proportional to F_2
c
c
      subroutine currnt(qc,omegac,pp,thetap,
     &           p,theta,itype,irel,sj)
c
      implicit none
c
      real*8 qc,omegac,pp,thetap,p,theta,q2,q,
     &      fac1,fac2,fac3,fac4,fac5,fac6,del1,del2,del3,sj,UCOmega,
     &      gp,etap,gam3,gpp,ep,fac,etapp,epp 
c
      dimension  sj(0:1,0:1,0:1,0:1,0:1,0:1),del1(0:1,0:1),
     &          del2(0:1,0:1),del3(0:1,0:1),UCOmega(0:1,0:1,0:1,0:1,4)
c
      integer itype,irel,rhop,rho,inrgyp,inrgy,lnuc,lnucp,ip,ipp
c
      real*8 mn,mt
      common/dpara/mn,mt
c
c               calculate square of four-momentum transfer
c
      q2=qc**2-omegac**2
      q=sqrt(q2)
c
      fac1=qc/q
      fac2=omegac/q
      fac3=0.5d0*q/mn
      fac4=1.0d0
      fac5=0.5d0*omegac/mn
      fac6=0.5d0*qc/mn

c
      call angfac(thetap,theta,del1,del2,del3)
      call DiracToPauli(pp,p,mn,UCOmega)
c      print *,UCOmega
c
      if(irel.eq.1) then                ! relativistic current operator
c
        do inrgyp=0,1
          rhop=1-2*inrgyp
c
          do inrgy=0,1
            rho=1-2*inrgy
c
              do lnuc=0,1  
                ip=abs(itype-lnuc)
c
                   do lnucp=0,1
                     ipp=abs(itype-lnucp)     
c
                     sj(lnucp,lnuc,inrgyp,inrgy,0,0)=
     &                 (fac1*UCOmega(lnucp,lnuc,inrgyp,inrgy,2)
     &                   *del1(ipp,ip)
     &                 -fac2*UCOmega(lnucp,lnuc,inrgyp,inrgy,4)
     &                   *del2(ipp,ip))
c
                     sj(lnucp,lnuc,inrgyp,inrgy,0,1)=
     &                 fac3*UCOmega(lnucp,lnuc,inrgyp,inrgy,3)
     &                   *del2(ipp,ip)
c                                                                      
                     sj(lnucp,lnuc,inrgyp,inrgy,1,0)=
     &                 -fac4*UCOmega(lnucp,lnuc,inrgyp,inrgy,4)
     &                   *del3(ipp,ip)
c
                     sj(lnucp,lnuc,inrgyp,inrgy,1,1)=
     &                 -(fac5*UCOmega(lnucp,lnuc,inrgyp,inrgy,3)
     &                 +fac6*UCOmega(lnucp,lnuc,inrgyp,inrgy,1))
     &                   *del3(ipp,ip)
c
              end do
            end do
          end do
        end do
c
      else if(irel.eq.0) then         ! nonrelativistic current operator
c
        epp=sqrt(pp**2+mn**2)
        ep=sqrt(p**2+mn**2)
        fac=sqrt((ep+mn)*(epp+mn))/(2.*mn)
        etapp=pp/(2.d0*mn)
        etap=p/(2.d0*mn)
c
        do lnuc=0,1
          gp=(1.d0-2.d0*lnuc)*etap
          ip=abs(itype-lnuc)
          do lnucp=0,1
            gpp=(1.d0-2.d0*lnucp)*etapp
            ipp=abs(itype-lnucp)
c
            gam3=gpp+gp
c
            sj(lnucp,lnuc,0,0,0,0)=fac1*del1(ipp,ip)
            sj(lnucp,lnuc,0,0,0,1)=0.0
c
            sj(lnucp,lnuc,0,1,0,0)=0.0
            sj(lnucp,lnuc,0,1,0,1)=0.0
c
            sj(lnucp,lnuc,1,0,0,0)=0.0
            sj(lnucp,lnuc,1,0,0,1)=0.0
c
            sj(lnucp,lnuc,1,1,0,0)=0.0
            sj(lnucp,lnuc,1,1,0,1)=0.0
c
            sj(lnucp,lnuc,0,0,1,0)=-fac4*gam3*del3(ipp,ip)
            sj(lnucp,lnuc,0,0,1,1)=-fac6*del3(ipp,ip)
c
            sj(lnucp,lnuc,0,1,1,0)=0.0
            sj(lnucp,lnuc,0,1,1,1)=0.0
c
            sj(lnucp,lnuc,1,0,1,0)=0.0
            sj(lnucp,lnuc,1,0,1,1)=0.0
c
            sj(lnucp,lnuc,1,1,1,0)=0.0
            sj(lnucp,lnuc,1,1,1,1)=0.0
          end do
        end do
c
      endif
c
      return
      end
c
c___________________________________________________________________________
c
c*****DiracToPauli
c
      subroutine DiracToPauli(pp,p,mn,UCOmega)
c
      implicit none
c
      real*8 pp,p,mn,epp,ep,fac,etapp,etap,gp,gpp,gam1,gam2,gam3,gam4,
     &       UCOmega(0:1,0:1,0:1,0:1,4)
c
      integer lnuc,lnucp
c
      epp=sqrt(pp**2+mn**2)
      ep=sqrt(p**2+mn**2)
      fac=sqrt((ep+mn)*(epp+mn))/(2.d0*mn)
      etapp=pp/(epp+mn)
      etap=p/(ep+mn)
c
      do lnuc=0,1
        gp=(1.d0-2.d0*lnuc)*etap
        do lnucp=0,1
          gpp=(1.d0-2.d0*lnucp)*etapp
c
          gam1=fac*(1.d0+gpp*gp)
          gam2=fac*(1.d0-gpp*gp)
          gam3=fac*(gpp+gp)
          gam4=fac*(gpp-gp)
c
          UCOmega(lnucp,lnuc,0,0,1)=gam2
          UCOmega(lnucp,lnuc,0,1,1)=-gam4
          UCOmega(lnucp,lnuc,1,0,1)=gam4
          UCOmega(lnucp,lnuc,1,1,1)=-gam2
c
          UCOmega(lnucp,lnuc,0,0,2)=gam1
          UCOmega(lnucp,lnuc,0,1,2)=gam3
          UCOmega(lnucp,lnuc,1,0,2)=gam3
          UCOmega(lnucp,lnuc,1,1,2)=gam1
c
          UCOmega(lnucp,lnuc,0,0,3)=-gam4
          UCOmega(lnucp,lnuc,0,1,3)=gam2
          UCOmega(lnucp,lnuc,1,0,3)=-gam2
          UCOmega(lnucp,lnuc,1,1,3)=gam4
c
          UCOmega(lnucp,lnuc,0,0,4)=gam3
          UCOmega(lnucp,lnuc,0,1,4)=gam1
          UCOmega(lnucp,lnuc,1,0,4)=gam1
          UCOmega(lnucp,lnuc,1,1,4)=gam3
c
        end do
c
      end do
c
      return
      end
      

