c  This file contains:
c    sachs,dircff,ftog,ftofp,gtofp,hohler,GariK,ijl,dipole,GariKrumpelmann
c
c_______________________________________________________________________
c
c*****sachs
c
      subroutine sachs(q2,g,iff)
c
c     This subroutine returns Sachs form factors for a variety of
c     parameterizations.
c
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c             iff=1..........Hohler 8.2 parameterization with renormaliztion
c                            according to Friar.
c                =2..........Gari and Krumpelmann
c                =3..........Iachello, Jackson and Lande
c                =4..........dipole parameterization, Galster
c                =5..........dipole parameterization, F_1 neutron=0
c                =6..........dipole parameteriztion by Cioffi degli Atti
c                =7..........dipole G_E neutron=0.0
c                =8..........Gari and Krumpelmann, model 1
c                =9..........Gari and Krumpelmann, model 2
c                =10.........Gari and Krumpelmann, model 3
c                =11.........Mergell, Meissner and Drechsel
c                =12.........MMDA Alternate three-pole parameterization
c                =13.........Ulf-G Meissner, V. Mull, J. Speth and J. W. Van Orden,
c                            Phys. Lett. B 408, 381 (1997).
c                =14.........VO, Five pole parameterization with all parameters floating,
c                            best fit to old data.
c                =15.........Five pole parameterization that fits new preliminary CEBAF
c                            G_E^p data
c                =16         old MMD 
c
c     OUTPUT
c             g(iso,itype)...form factors
c
c             iso=0..........proton
c                 1..........neutron
c             itype=0........electric form factor
c                  =1........magnetic form factor
c
      implicit real*8 (a-h,o-z)
c
      dimension g(0:1,0:1),f(2,2)
c
      if(iff.eq.1) then
        call hohler(q2,f)
        call ftog(q2,f,g)
      else if(iff.eq.2) then
        call garik(q2,f)
        call ftog(q2,f,g)
      else if(iff.eq.3) then
        call ijl(q2,f)
        call ftog(q2,f,g)
      else if(iff.ge.4.and.iff.le.7) then
        iffm3=iff-3
        call dipole(q2,g,iffm3)
      else if(iff.ge.8.and.iff.le.10) then
        iffm7=iff-7
        call GariKrumpelmann(q2,iffm7,f)
        call ftog(q2,f,g)
      else if(iff.ge.11.and.iff.le.15) then
	  iffm10=iff-10
        call mmd(q2,iffm10,f)
        call ftog(q2,f,g)
	else if(iff.eq.16) then
	  call mmd_old(q2,f)
      else
        print *,' unknown nucleon form factor type, iff=',iff
      endif
c
      return
      end
c
c_______________________________________________________________________
c
c*****dircff
c
      subroutine dircff(q2,fp,iff)
c
c     This subroutine returns Dirac and Pauli form factors for a variety of
c     parameterizations.
c
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c             iff=-2.........F_2p=1 all others =0
c                =-1.........F_1p=1 all others =0
c                =0..........all =0
c                =1..........Hohler 8.2 parameterization with renormaliztion
c                            according to Friar.
c                =2..........Gari and Krumpelmann
c                =3..........Iachello, Jackson and Lande
c                =4..........dipole parameterization
c                =5..........dipole parameterization, F_1 neutron=0
c                =6..........dipole parameteriztion by Cioffi degli Atti
c                =7..........dipole G_E neutron=0.0
c                =8..........Gari and Krumpelmann, model 1
c                =9..........Gari and Krumpelmann, model 2
c                =10.........Gari and Krumpelmann, model 3
c                =11.........Mergell, Meissner and Drechsel
c                =12.........MMDA Alternate three-pole parameterization
c                =13.........Ulf-G Meissner, V. Mull, J. Speth and J. W. Van Orden,
c                            Phys. Lett. B 408, 381 (1997).
c                =14.........VO, Five pole parameterization with all parameters floating,
c                            best fit to old data.
c                =15.........Five pole parameterization that fits new preliminary CEBAF
c                            G_E^p data
c                =16         old MMD 
c                
c
c     OUTPUT
c             fp(iso,itype)..form factors
c
c             iso=0..........proton
c                 1..........neutron
c             itype=1........Dirac form factor
c                  =2........Pauli form factor
c
      implicit real*8 (a-h,o-z)
c
      dimension g(0:1,0:1),f(2,2),fp(0:1,2)
c
      if(iff.eq.-2) then
        fp(0,1)=0.
        fp(0,2)=1.
        fp(1,1)=0.
        fp(1,2)=0.
      else if(iff.eq.-1) then
        fp(0,1)=1.
        fp(0,2)=0.
        fp(1,1)=0.
        fp(1,2)=0.
      else if(iff.eq.0) then
        fp(0,1)=0.
        fp(0,2)=0.
        fp(1,1)=0.
        fp(1,2)=0.
      else if(iff.eq.1) then
        call hohler(q2,f)
        call ftofp(f,fp)
      else if(iff.eq.2) then
        call garik(q2,f)
        call ftofp(f,fp)
      else if(iff.eq.3) then
        call ijl(q2,f)
        call ftofp(f,fp)
      else if(iff.ge.4.and.iff.le.7) then
        iffm3=iff-3
        call dipole(q2,g,iffm3)
        call gtofp(q2,g,fp)
      else if(iff.ge.8.and.iff.le.10) then
        iffm7=iff-7
        call GariKrumpelmann(q2,iffm7,f)
        call ftofp(f,fp)
      else if(iff.ge.11.and.iff.le.15) then
	  iffm10=iff-10
        call mmd(q2,iffm10,f)
        call ftofp(f,fp)
	else if(iff.eq.16) then
	  call mmd_old(q2,f)
        call ftofp(f,fp)
      else
        print *,' unknown nucleon form factor type, iff=',iff
      endif
c
      return
      end

c
c_______________________________________________________________________
c
c*****ftog
c
      subroutine ftog(q2,f,g)
c
c     This subroutine translates the representation of Dirac
c     form factors to Sachs form factors.
c
      implicit real*8(a-h,o-z)
c
      real*8 mn
c
      dimension f(2,2),g(0:1,0:1),fp(0:1,2)
c
      data mn/.9389/
c
      tau=q2/(4.*mn**2)
c
      call ftofp(f,fp)
c
      do 100 i=0,1
        g(i,0)=fp(i,1)-tau*fp(i,2)
        g(i,1)=fp(i,1)+fp(i,2)
 100  continue
c 
      return
      end
c
c_______________________________________________________________________
c
c*****ftofp
c
      subroutine ftofp(f,fp)
c
c     This subroutine translates isoscalar and isovector Dirac form 
c     factors to neutron and proton form factors.
c
      implicit real*8(a-h,o-z)
c
      dimension f(2,2),fp(0:1,2)
c
      do 100 i=1,2
        fp(0,i)=0.5*(f(2,i)+f(1,i))
        fp(1,i)=0.5*(f(2,i)-f(1,i))
 100  continue
c
      return
      end
c
c_______________________________________________________________________
c
c*****gtofp
c
      subroutine gtofp(q2,g,fp)
c
c     This subroutine translates Sachs form factors to Dirac form
c     factors.

      implicit real*8(a-h,o-z)
c
      real*8 mn
c
      dimension g(0:1,0:1),fp(0:1,2)
c
      data mn/.9389/
c
      tau=q2/(4.*mn**2)
c
      do 100 i=0,1
        fp(i,1)=(g(i,0)+tau*g(i,1))/(1.0+tau)
        fp(i,2)=(g(i,1)-g(i,0))/(1.+tau)
 100  continue
c
      return
      end
c
c_______________________________________________________________________
c
c*****hohler
c
      subroutine hohler(q2,f)
c
c     This subroutine calculates the electromagnetic form factors
c     using the Hohler parameterization 8.2.  Input is in GeV**2
c     and the square of the four-momentum transfer should be
c     positive.
c
c     REFERENCE: G. Hohler, et al., Nucl. Phys. B114, 505 (1976).
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c
c     OUTPUT
c             f(iso,itype)...form factors
c
c             iso=1..........isovector
c                 2..........isoscalar
c             itype=1........Dirac form factor F_1
c                  =2........Pauli form factor F_2
c
c     7-16-87
c            This version has been modified such that it takes into
c            account changes made by J. Friar to this parameterization
c            to give better behavior at the q2=0 point.
c
      implicit real*8(a-h,k-z)
      dimension srt(3,2),a(3,2,2),f(2,2),frov(2)
      data((srt(i,j),i=1,3),j=1,2)/1.21,2.45,2.95,0.783,1.02,1.8/
      data(((a(i,j,ij),i=1,3),j=1,2),ij=1,2)/0.05,-0.52,0.28,0.71
     #  ,-0.64,-0.13,-1.99,0.1956,0.1856,-0.11,0.13,-0.02/
c
      frov(1)=(0.955+0.09/(1.+q2/0.355)**2)/(1.+q2/0.536)/2.
      frov(2)=(5.335+0.962/(1.+q2/0.268))/(1.+q2/.603)/2.
c
      do 50 i1=1,2
        do 50 i2=1,2
          f(i1,i2)=frov(i2)*(1-(-1)**i1)/2.
          do 50 i3=1,3
50          f(i1,i2)=f(i1,i2)+a(i3,i1,i2)/(srt(i3,i1)**2+q2)
c
c                  Renormalization according to Friar
c
      f(1,1)=2.*f(1,1)*0.9954211
      f(2,1)=2.*f(2,1)*0.9930881
      f(1,2)=2.*f(1,2)*1.0052166
      f(2,2)=2.*f(2,2)*0.98891518
c
      return
      end
c
c_______________________________________________________________________
c
c*****GariK
c
      subroutine garik(q2,f)
c
c     This subroutine produces the form factors of Gari and Krumpelmann.
c
c
c     REFERENCE: M. Gari and W. Krumpelmann, Z. Phys. A322 (1985);
c                                            Phys. Lett. 173B, 10 (1986).
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c
c     OUTPUT
c             f(iso,itype)...form factors
c
c             iso=1..........isovector
c                 2..........isoscalar
c             itype=1........Dirac form factor F_1
c                  =2........Pauli form factor F_2
c
      implicit real*8(a-h,o-z)
c
      real*8 mrho,momega,kapv,kaps,kaprho,kapomg,lam1,lam2,lamqcd
c
      dimension f(2,2)
c
      data mrho/0.7661/,momega/0.784/,kapv/3.706/,kaps/-0.12/,
     &     kaprho/6.62/,kapomg/0.163/,crho/0.377/,comg/0.411/,
     &     lam1/0.795/,lam2/2.27/,lamqcd/0.29/
c
      q2tild=q2*log((lam2**2+q2)/lamqcd**2)/log((lam2/lamqcd)**2)
c
      d1=1./(1.+q2tild/lam1**2)
      d2=1./(1.+q2tild/lam2**2)
c
      f1=d1*d2
      f2=f1*d2
c
      drho=1./(1.+q2/mrho**2)
      domg=1./(1.+q2/momega**2)
c
      f(1,1)=(crho*drho+1.-crho)*f1
      f(2,1)=(comg*domg+1.-comg)*f1
      f(1,2)=(crho*kaprho*drho+kapv-crho*kaprho)*f2
      f(2,2)=(comg*kapomg*domg+kaps-comg*kapomg)*f2
c
      return
      end
c
c
c_______________________________________________________________________
c
c*****ijl
c
      subroutine ijl(q2,f)
c
c     This subroutine produces the form factors of Iachello, Jackson and
c     Lande.
c
c     REFERENCE: F. Iachello, A. D. Jackson and A. Lande, Phys. Lett. 43B,
c                191 (1973).
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c
c     OUTPUT
c             f(iso,itype)...form factors
c
c             iso=1..........isovector
c                 2..........isoscalar
c             itype=1........Dirac form factor F_1
c                  =2........Pauli form factor F_2
c
      implicit real*8(a-h,o-z)
c
      real*8 mpi,mrho,momega,mphi,kapv,kaps
c
      dimension f(2,2)
c
      data mpi/.13957/,mrho/0.765/,momega/0.784/,mphi/1.019/,
     &     gamrho/.112/,gam/0.25/,btarho/0.672/,btaomg/1.102/,
     &     btaphi/0.112/,alpphi/-0.052/,kaps/-0.120/,kapv/3.706/,
     &     pi/3.141592741012573/
c
      g=1./(1.+gam*q2)**2
c
      if(q2.gt.1.e-7) then
        alpha=2./pi*sqrt((q2+4.*mpi**2)/q2)*log((sqrt(q2+4.*mpi**2)+
     &        sqrt(q2))/(2.*mpi))
      else
        alpha=0.
      endif
c
      drho=(mrho**2+8.*gamrho*mpi/pi)/(mrho**2+q2+(4.*mpi**2+q2)*
     &gamrho*alpha/mpi)
      domg=1./(1.+q2/momega**2)
      dphi=1./(1.+q2/mphi**2)
c
      f(1,1)=g*(1.-btarho+btarho*drho)
      f(2,1)=g*(1-btaomg-btaphi+btaomg*domg+btaphi*dphi)
      f(1,2)=g*kapv*drho
      f(2,2)=g*((kaps-alpphi)*domg+alpphi*dphi)
c
      return
      end
c
c_______________________________________________________________________
c
c*****dipole
c
      subroutine dipole(q2,g,iff)
c
c     This subroutine produces several form factors which are expressed
c     as dipole fits to the Sachs form factors.
c
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c             iff=1..........Galster
c                =2..........F_1 neutron=0
c                =3..........Cioffi degli Atti
c                =4..........G_E neutron=0
c
c     OUTPUT
c             g(iso,itype)...form factors
c
c             iso=0..........proton
c                 1..........neutron
c             itype=0........electric form factor
c                  =1........magnetic form factor
c
      implicit real*8(a-h,o-z)
c
      real*8 mn,mup,mun
c
      dimension g(0:1,0:1)
c
      data mn/0.9389/,mup/2.793/,mun/-1.913/
c
      Tau=q2/(4.*mn**2)
c
c                          Galster 
c               S. Galster, et al., Nucl. Phys. B32, 221 (1971).
c
      if(iff.eq.1) then
        G(0,0)=1./(1.+q2/0.71)**2
        G(1,0)=-mun*Tau*G(0,0)/(1.+5.6*Tau)   !5.6
        G(0,1)=mup*G(0,0)
        G(1,1)=mun*G(0,0)
c
c           dipole with neutron Dirac form factor set to zero
c
      else if(iff.eq.2) then
        G(0,0)=1./(1.+q2/0.71)**2
        G(1,0)=-mun*Tau*G(0,0)
        G(0,1)=mup*G(0,0)
        G(1,1)=mun*G(0,0)
c
c           dipole fit according to Cioffi degli Atti
c
      else if(iff.eq.3) then
        G(0,0)=1./(1.+q2/0.695)**2
        G(1,0)=-mun*Tau*G(0,0)/(1.+4.*Tau)
        G(0,1)=mup*G(0,0)
        G(1,1)=mun*G(0,0)
c
c                     G_E neutron = 0.
c
      else if(iff.eq.4) then
        G(0,0)=1./(1.+q2/0.71)**2
        G(1,0)=0.0
        G(0,1)=mup*G(0,0)
        G(1,1)=mun*G(0,0)
      endif
      return
      end
c
c_______________________________________________________________________
c
c*****GariKrumpelmann
c
      subroutine GariKrumpelmann(q2,i_version,f)
c
c     This subroutine produces the form factors of Gari and Krumpelmann.
c     This includes all versions in the second set of references
c
c
c     REFERENCE: M. Gari and W. Krumpelmann, Z. Phys. A322 (1985);
c                                            Phys. Lett. 173B, 10 (1986);
c
c                                            Phys. Lett. B274, 159 (1992);
c                                            Phys. Lett. B282, 483 (1992).
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c             i_version......1 to 3 corresponding to notation in 1992 
c                            papers.
c
c     OUTPUT
c             f(iso,itype)...form factors
c
c             iso=1..........isovector
c                 2..........isoscalar
c             itype=1........Dirac form factor F_1
c                  =2........Pauli form factor F_2
c
c      implicit real*8(a-h,o-z)
      implicit none
c
      real*8 m_rho,m_omega,m_phi,kappa_v,kappa_s,
     &       c_rho(3),kappa_rho(3),c_omega(3),kappa_omega(3),
     &       c_phi(3),kappa_phi(3),mu_phi(3),lambda_1(3),
     &       lambdaD_1(3),lambda_2(3),lambda_qcd(3),f(2,2)
c
      real*8 q2,q2_tilda,d1,d1d,d2,f1_rho,f1_omega,f1_D,
     &       f1_phi,f2_rho,f2_omega,f2_D,f2_phi,d_rho,
     &       d_omega,d_phi
c
      integer i_version
c
      data m_rho/0.766/,m_omega/0.784/,m_phi/1.019/,
     &     kappa_v/3.706/,kappa_s/-0.12/,
     &     c_rho/0.377,0.5927,0.5688/,
     &     kappa_rho/6.62,3.425,3.642/,
     &     c_omega/0.411,0.6212,0.5774/,
     &     kappa_omega/0.163,0.3941,0.4775/,
     &     c_phi/0.0,0.0,-0.666/,
     &     kappa_phi/0.0,0.0,-0.2378/,
     &     mu_phi/0.33,0.33,0.33/,
     &     lambda_1/0.795,0.867,0.823/,
     &     lambdaD_1/0.795,1.194,1.24/,
     &     lambda_2/2.270,2.063,1.95/,
     &     lambda_qcd/0.29,0.344,0.31/
c
      q2_tilda=q2*log((lambda_2(i_version)**2+q2)/
     &                lambda_qcd(i_version)**2)/
     &            log((lambda_2(i_version)/
     &                lambda_qcd(i_version))**2)
c
      d1=1./(1.+q2_tilda/lambda_1(i_version)**2)
      d1d=1./(1.+q2_tilda/lambdaD_1(i_version)**2)
      d2=1./(1.+q2_tilda/lambda_2(i_version)**2)
c
      f1_rho=d1*d2
      f1_omega=f1_rho
      f1_D=d1d*d2
      f1_phi=f1_rho*sqrt((q2/(lambda_1(i_version)**2+q2))**3)
c
      if(i_version.eq.1) then
        f2_rho=d2*f1_rho
        f2_D=d2*f1_D
      else 
        f2_rho=d1*f1_rho
        f2_D=d1d*f1_D
      end if
      f2_omega=f2_rho
c
      f2_phi=f2_rho*sqrt(((1.+q2/mu_phi(i_version)**2)/
     &       (1.+q2/lambda_1(i_version)**2))**3)
c
      d_rho=1./(1.+q2/m_rho**2)
      d_omega=1./(1.+q2/m_omega**2)
      d_phi=1./(1.+q2/m_phi**2)
c
      f(1,1)=c_rho(i_version)*d_rho*f1_rho
     &      +(1.0-c_rho(i_version))*f1_D
      f(2,1)=c_omega(i_version)*d_omega*f1_omega
     &      +c_phi(i_version)*d_phi*f1_phi
     &      +(1.0-c_omega(i_version))*f1_D
      f(1,2)=kappa_rho(i_version)*c_rho(i_version)*
     &        d_rho*f2_rho
     &      +(kappa_v-kappa_rho(i_version)*
     &        c_rho(i_version))*f2_D
      f(2,2)=kappa_omega(i_version)*c_omega(i_version)*
     &        d_omega*f2_omega
     &      +kappa_phi(i_version)*c_phi(i_version)*
     &        d_phi*f2_phi
     &      +(kappa_s-kappa_omega(i_version)*
     &        c_omega(i_version)
     &       -kappa_phi(i_version)*c_phi(i_version))*f2_D
c
      return
      end
c_______________________________________________________________________
c
c*****MMD_old
c
      subroutine mmd_old(q2,f)
c
c     This subroutine produces the for factors of Mergell, Meissner
c     and Drechsel.
c
c     REFERENCE P. Mergell, Ulf-G. Meissner and D. Drechsel, Nucl. Phys.
c               A 596, 367 (1996).
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c
c     OUTPUT
c             f(iso,itype)...form factors
c
c             iso=1..........isovector
c                 2..........isoscalar
c             itype=1........Dirac form factor F_1
c                  =2........Pauli form factor F_2
c
      implicit real*8(a-h,o-z)
c
      dimension f(2,2)
c
      flna=log((9.73+q2)/0.350)**(-2.148)
c
      f(1,1)=2.*((13.16596+1.1245/(1+q2/0.318)**2)/(2.*(1+q2/0.550))
     &       -38.868/(2.103+q2)+425.339/(2.723+q2)
     &       -388.651/(2.856+q2))*flna
      f(1,2)=2.*((73.7917+5.0212/(1+q2/0.142))/(2.*(1+q2/0.5360))
     &       -73.495/(2.103+q2)+83.261/(2.723+q2)
     &       -29.394/(2.856+q2))*flna
      f(2,1)=2.*(9.457/(0.612+q2)-9.050/(1.038+q2)
     &        -0.410/(2.560+q2))*flna
      f(2,2)=2.*(-1.545/(0.612+q2)+1.987/(1.038+q2)
     &        -0.436/(2.560+q2))*flna
c
c      write(6,*) f
      return
      end
c
c_______________________________________________________________________
c
c*****MMD
c
      subroutine mmd(q2,i_version,f)
c
c     This subroutine produces the for factors of Mergell, Meissner
c     and Drechsel.
c
c     REFERENCE P. Mergell, Ulf-G. Meissner and D. Drechsel, Nucl. Phys.
c               A 596, 367 (1996).
c
c     INPUT
c             Q2.............negative of square of four-momentum transfer
c                            in Gev**2. Therefore Q2>0 for spacelike
c                            momentum transfers.
c             i_version=1....MMD parameterization. This does not use the exact parameters
c                            from the reference above since they are do not reliably
c                            reproduce the fit and constraints.
c                      =2....Alternate three-pole parameterization
c                      =3....Ulf-G Meissner, V. Mull, J. Speth and J. W. Van Orden,
c                            Phys. Lett. B 408, 381 (1997).
c                      =4....Five pole parameterization with all parameters floating,
c                            best fit to old data.
c                      =5....Five pole parameterization that fits new preliminary CEBAF
c                            G_E^p data 
c
c     OUTPUT
c             f(iso,itype)...form factors
c
c             iso=1..........isovector
c                 2..........isoscalar
c             itype=1........Dirac form factor F_1
c                  =2........Pauli form factor F_2
c
      implicit none
c
      real*8 q2,deep_fint(0:1,2),f(2,2),fln,factor          !,g(2)
c
      integer i,n,iso,iexp,i_version
c
      integer max_terms,n_terms(0:1,5)
c
      parameter(max_terms=5)
c
      real*8 c_s,Q0Sqrd,LambdaSqrd,arho,brho,
     &       crho,drho,MaSqrd,MbSqrd,gamma
c
      common/model_parameters/Q0Sqrd,LambdaSqrd(5),
     &                        c_s(max_terms,0:2,0:1,5),
     &                        arho(2),brho(2),crho(2),drho(2),MaSqrd,
     &                        MbSqrd
      parameter(gamma=2.148)
c
      data MaSqrd/0.5/,MbSqrd/0.4/,Q0Sqrd/0.35/
      data arho/1.0317,5.7824/
      data brho/0.0875,0.3907/
      data crho/0.3176,0.1422/
      data drho/0.5496,0.5362/
	data LambdaSqrd(1)/9.733/
      data (n_terms(iso,1),iso=0,1)/3,3/
      data ((c_s(n,i,0,1),i=0,2),n=1,3)/
     &      6.115240E-01,   7.469750E-01,  -1.221656E-01,
     &      1.038361E+00,  -7.361827E-01,   1.609093E-01,
     &      2.585406E+00,  -3.998969E-02,  -3.983142E-02/
      data ((c_s(n,i,1,1),i=0,2),n=1,3)/
     &      2.102500E+00,  -3.414994E+00,  -6.544095E+00,
     &      2.734393E+00,   3.099864E+01,   7.134224E+00,
     &      2.868281E+00,  -2.809906E+01,  -2.081179E+00/
c
	data LambdaSqrd(2)/7.1549/
      data (n_terms(iso,2),iso=0,1)/3,3/
      data ((c_s(n,i,0,2),i=0,2),n=1,3)/
     &      6.115240E-01,   7.368497E-01,  -1.106676E-01,
     &      1.038361E+00,  -7.204214E-01,   1.310221E-01,
     &      4.422851E+00,  -1.104352E-01,  -2.986785E-02/
      data ((c_s(n,i,1,2),i=0,2),n=1,3)/
     &      2.102500E+00,  -4.904924E+00,  -5.681093E+00,
     &      2.734393E+00,   4.603449E+01,  -9.500462E-01,
     &      2.868281E+00,  -4.203599E+01,   5.292896E+00/
c
      data LambdaSqrd(3)/9.733/
      data (n_terms(iso,3),iso=0,1)/4,3/
      data ((c_s(n,i,0,3),i=0,2),n=1,4)/
     &      6.115240E-01,   6.773855E-01,  -9.657211E-02,
     &      1.038361E+00,  -1.930000E-02,  -4.000000E-03,
     &      1.254400E+00,  -7.987780E-01,   1.525659E-01,
     &      2.656775E+00,   1.215471E-01,  -5.340202E-02/
      data ((c_s(n,i,1,3),i=0,2),n=1,3)/
     &      2.102500E+00,  -3.607208E+00,  -6.534883E+00,
     &      2.722500E+00,   2.017498E+01,   5.946551E+00,
     &      2.958706E+00,  -1.710495E+01,  -9.027768E-01/
c
      data LambdaSqrd(4)/9.733/
      data (n_terms(iso,4),iso=0,1)/5,5/
	data ((c_s(n,i,0,4),i=0,2),n=1,5)/
     &      6.365155E-01,   7.816823E-01,   2.282614E-01,
     &      1.307334E+00,  -2.226998E-02,   9.921241E-02,
     &      1.225704E+00,  -5.698303E-01,   3.458286E+00,
     &      2.580973E+00,   1.073272E-01,  -2.756912E-01,
     &      1.095059E+00,  -3.173048E-01,  -3.514883E+00/
	data ((c_s(n,i,1,4),i=0,2),n=1,5)/
     &      2.432420E+00,   5.649976E+01,  -6.387368E+00,
     &      2.560801E+00,  -4.520188E+01,   7.035124E+00,
     &      2.072874E+00,  -1.159112E+01,  -4.658646E+00,
     &      3.038462E+00,  -8.905151E-02,   4.109003E+00,
     &      3.497939E+00,  -1.399468E-01,  -1.594393E+00/
c
      data LambdaSqrd(5)/9.733/
      data (n_terms(iso,5),iso=0,1)/5,5/
	data ((c_s(n,i,0,5),i=0,2),n=1,5)/
     &      6.780836E-01,   1.030995E+00,   2.596098E-01,
     &      1.138074E+00,  -2.295158E-01,   1.617209E+00,
     &      1.734596E+00,  -5.213922E-01,  -7.919872E-02,
     &      3.215552E+00,   3.100522E-01,  -8.486819E-02,
     &      9.577866E-01,  -5.884375E-01,  -1.716670E+00/
	data ((c_s(n,i,1,5),i=0,2),n=1,5)/
     &      1.597234E+00,  -1.573959E-01,  -7.707051E+00,
     &      1.916871E+00,   6.384136E-01,   6.882814E+00,
     &      8.032118E-01,  -8.592565E-02,   3.692524E-01,
     &      3.708225E+00,   7.088970E-02,  -6.981130E+00,
     &      4.195078E+00,  -9.613039E-01,   6.031879E+00/
c
      factor=fln(Q2,i_version)
c
      do iso=0,1
c
        do i=1,2
c
          deep_fint(iso,i)=0.0
c
          do n=1,n_terms(iso,i_version)
            deep_fint(iso,i)=deep_fint(iso,i)+c_s(n,i,iso,i_version)
     &    	  /fln(-c_s(n,0,iso,i_version),i_version)/
     &               (c_s(n,0,iso,i_version)+q2)
          end do
c
          deep_fint(iso,i)=deep_fint(iso,i)*factor
c
        end do
c
      end do
c
      do i=1,2
        iexp=3-i
        deep_fint(1,i)=deep_fint(1,i)+0.5d0*factor*(arho(i)/fln(-MaSqrd,i_version)
     &         +brho(i)/fln(-MbSqrd,i_version)/
     &         (1.d0+q2/crho(i))**iexp)/(1.d0+q2/drho(i))
      end do
c
      do iso=0,1
	  do i=1,2
	    f(2-iso,i)=2.d0*deep_fint(iso,i)
	  end do
	end do
c
      return
      end
c
c_______________________________________________________________________
c
c*****fln.f
c
      function fln(Q2,i_version)
c
      implicit none
c
      real*8 fln,Q2
c
      integer max_terms,i_version
c
      parameter(max_terms=5)
c
      real*8 c_s,Q0Sqrd,LambdaSqrd,arho,brho,
     &       crho,drho,MaSqrd,MbSqrd,gamma
c
      common/model_parameters/Q0Sqrd,LambdaSqrd(5),
     &                        c_s(max_terms,0:2,0:1,5),
     &                        arho(2),brho(2),crho(2),drho(2),MaSqrd,
     &                        MbSqrd
      parameter(gamma=2.148)
c      
      fln=log((LambdaSqrd(i_version)+Q2)/Q0Sqrd)**(-gamma)
c
      return
      end
